#!/bin/sh
#
# 이 도구는 Apache, MySQL, PHP 소스를 다운로드하여 자동으로 컴파일하여
# 설치해 주는 AMP 설치 도구(AMP Installer)입니다.
#
# 이 도구는 가장 최신의 Apache와 MySQL 소스를 소스 서버를 체크하여
# 보고해 주며, 이 가운에서 원하는 버전을 선택하여 자동으로 컴파일해 줍니다.
# PHP 소스는 이 스크립트가 작성된 시점에서 지원하는 버전만 지원합니다.
#
# 이 도구는 기본적으로 AMP 소스가 컴파일될 수 있는 환경이 갖춰진 시스템에서만
# 원활하게 동작하므로, 여러분의 서버가 최적의 컴파일 환경이 구비되어 있는지를
# 체크해 보셔야 합니다.
#
# 이 도구는 프로그래밍 능력이 없는 상태에서 작성되었기 때문에 스크립트 내용이
# 좀 조잡할 수 있으며, 불필요한 부분도 없지 않습니다. 누구나 쉽게 APM를 설치하여
# 사용할 수 있도록 하기 위해서 만들어진 것으로 잘못된 부분이나 올바르게 수정
# 해야 하는 부분이 있다면 스크립터를 지도해 주시면 감사하겠습니다.
#
# 스크립터 : Zharyong Suh (no1linuxer@no1linux.org)
# 일자: 2010년 3월 1일
#

. /etc/init.d/functions

if [ "$UID" != "0" ]; then
        gprintf "이 스크립트는 root 권한으로 실행해야 합니다."; echo
        exit
fi

if [ -f /etc/sysconfig/network ]; then
    . /etc/sysconfig/network
    [ ${NETWORKING} = "no" ] && exit 0
fi

gprintf() {
        local TEXT=`gettext -e "$1"`
        shift
        printf "$TEXT" "$@"
}

# Check RPM pkg installed
apacheinstdir=/usr/local/apache2
apacheconfdir=$apacheinstdir/conf

# apache install scrypt
REMOTEDIR=net/apache/httpd
FTPHOST=ftp.riken.go.jp
WEBHOST=ftp.apache-kr.org
PORT=21
USER=anonymous
PASSWD=ftp@anonymous

    clear
    gprintf "먼저 Apache를 설치합니다."; echo

ftp -n -v $FTPHOST $PORT <<EOT > /tmp/apache-ftp.log
ascii
passive
user $USER $PASSWD
cd $REMOTEDIR/
ls httpd*
bye
EOT

apachever=`grep httpd /tmp/apache-ftp.log | awk '$9~/.*bz2/' | sed "s/.*httpd-//g" | sed "s/\.*tar.*//g" | uniq`

gprintf "설치 가능한 apache 소스 버전은 다음과 같습니다." ; echo
gprintf "========================="; echo
gprintf "    httpd 소스 버전      "; echo
gprintf "========================="; echo
gprintf "$apachever               "; echo
gprintf "========================="; echo
gprintf ""                         ; echo
echo -n "step1.설치하고자 하는 아파치 소스 버전을 선택해 주세요? "
read ver
    if [ -z $ver ]; then
    echo -n "아파치 소스 버전을 입력하지 않으셨습니다. 다시 입력해 주세요! "
    read key
    gprintf "아파치 소스 버전을 선택하지 않아 종료합니다."; echo
    exit 1

    elif [ ! -z $ver ]; then
    srcfile=httpd-$ver.tar.bz2
    gprintf "step2.아파치 소스를 /tmp/httpd-$ver 디렉토리로 설치하고 있습니다. 기다려 주세요....!"; echo
    wget http://$WEBHOST/httpd/$srcfile -P /tmp/ 2>/dev/null >/dev/null
    tar xvfj /tmp/$srcfile -C /tmp >/dev/null
    rm -f /tmp/$srcfile > /dev/null
fi

# 아파치 컴파일 환경 설정
cd /tmp/httpd-$ver
   gprintf "step3.컴파일 환경을 설정하고 있습니다. 기다려 주세요......!"; echo
./configure --prefix=$apacheinstdir --enable-mods-shared=all --with-mpm=prefork --enable-ssl --with-ssl --enable-so 2>/dev/null >/dev/null

if [ $? = "1" ]; then
    echo "컴파일 환경을 설정할 수 없습니다."
    gprintf "이는 컴파일에 필요한 환경 조건이 구비되지 않았으므로"; echo
    gprintf "컴파일 환경 조건을 먼저 해결해 주시기 바랍니다."; echo
    gprintf " ./configure 명령을 실행하여 오류가 발생되는 부분을 처리해 주세요." ; echo
    exit 1
fi

# 아파치 컴파일
    gprintf "step4.아파치를 컴파일하고 있는 중입니다. 기다려 주세요.........!"; echo
make -j4 2>/dev/null >/dev/null
if [ $? = "1" ]; then
    gprintf "컴파일에서 오류가 발생되었습니다."; echo
    gprintf "make 명령으로 오류를 확인하시고 처리해 주세요."; echo
    exit 1
fi

# 아파치 컴파일 설치
    gprintf "step5.컴파일한 파일들을 /usr/local/apache2 디렉토리로 설치하고 있습니다. 기다려 주세요.....!"; echo
make install 2>/dev/null >/dev/null
if [ $? = "1" ]; then
    gprintf "컴파일에서 오류가 발생되었습니다."; echo
    gprintf "make install 명령으로 오류를 확인하시고 처리해 주세요."; echo
    exit 1
fi

gprintf "step6. 아파치 일부 설정을 수정하고 있습니다. 기다려 주세요....!"; echo
# apache 버전 기록
if [ ! -f $apacheinstdir/.version ]; then
    echo $ver > $apacheinstdir/.version
fi

# apache 계정과 그룹 체크
apacheuser=`grep apache /etc/passwd | cut -d":" -f1 | awk '{print $1}'`

if [ apache != "$apacheuser" ]; then
   groupadd -g 75 apache
   adduser -u 75 -g apache -d $apacheinstdir apache
fi

# Apache Configuration
#
perl -pi -e "s/User daemon/User apache/g" $apacheconfdir/httpd.conf
perl -pi -e "s/Group daemon/Group apache/g" $apacheconfdir/httpd.conf
perl -pi -e "s/DirectoryIndex index.html/DirectoryIndex index.php index.html/g" $apacheconfdir/httpd.conf
perl -pi -e "s/Options Indexes FollowSymLinks/Options -Indexes FollowSymLinks/g" $apacheconfdir/httpd.conf
cp $apacheinstdir/bin/apachectl /etc/init.d/httpd >/dev/null
/etc/init.d/httpd restart >/dev/null

# ldconfig 재설정
cat >> /etc/ld.so.conf <<EOT
/usr/local/apache2/lib
/usr/local/apache2/modules
EOT
/sbin/ldconfig

# 소스 삭제
gprintf "step7.아파치 소스를 제거합니다."; echo
rm -rf /tmp/httpd-$ver >/dev/null
gprintf "step8.Apache $ver 버전이 성공적으로 설치되었습니다."; echo

sleep 4

    clear
    gprintf "이제 MySQL를 설치합니다......."; echo


# MySQL Install
FTPREMOTEDIR=pub/mysql/Downloads
FTPHOST=ftp.jaist.ac.jp
WEBREMOTEDIR=get/Downloads
WEBHOST=dev.mysql.com
PORT=21
USER=anonymous
PASSWD=ftp@anonymous

ftp -n -v $FTPHOST $PORT <<EOT > /tmp/mysql-ftp.log
user $USER $PASSWD
ascii
passive
ls pub/mysql/Downloads
ls pub/mysql/Downloads/MySQL-4.1/mysql-4.1*
ls pub/mysql/Downloads/MySQL-5.0/mysql-5.0*
ls pub/mysql/Downloads/MySQL-5.1/mysql-5.1*
ls pub/mysql/Downloads/MySQL-5.2/mysql-5.2*
ls pub/mysql/Downloads/MySQL-5.4/mysql-5.4*
ls pub/mysql/Downloads/MySQL-5.5/mysql-5.5*
bye
EOT

mysqlver=`grep mysql /tmp/mysql-ftp.log | awk '$9~/.*mysql-[0-5].[0-9].[0-9][0-9].tar*/' | sed "s/.*mysql-//g" | sed "s/\.*tar.*//g" | uniq`
#mysqlver50=`grep mysql /tmp/mysql-ftp.log | awk '$9~/.*mysql-5.0.[0-9][0-9].tar*/' | sed "s/.*mysql-//g" | sed "s/\.*tar.*//g" | uniq`
#mysqlver51=`grep mysql /tmp/mysql-ftp.log | awk '$9~/.*mysql-5.1.[0-9][0-9].tar*/' | sed "s/.*mysql-//g" | sed "s/\.*tar.*//g" | uniq`

gprintf "설치 가능한 MySQL 소스 버전은 다음과 같습니다."; echo
gprintf "================"; echo
gprintf " MySQL 소스 버전"; echo
gprintf "================"; echo
gprintf "$mysqlver       "; echo
gprintf "================"; echo

echo -n "step1.설치하고자 하는 MySQL 소스 버전을 선택해 주세요! "
read ver
instver=`echo "$ver" | sed "s/\.[0-9][0-9]//g"`

if [ -z $ver ]; then
        echo -n "MySQL 소스 버전을 입력하지 않으셨습니다. 다시 입력해 주세요! "
        read ver
        gprintf "MySQL 소스 버전을 선택하지 않아 종료합니다." ; echo
        exit 1

        elif [ ! -z $ver ]; then
                gprintf "step2.MySQL 소스를 다운로드하여 /tmp/mysql-$ver 디렉토리로 설치하고 있는 중입니다. 기다려 주세요......!"; echo
                srcfile=mysql-$ver.tar.gz
                mirrorsite=from/http://mysql.byungsoo.net/
                wget http://$WEBHOST/$WEBREMOTEDIR/MySQL-$instver/$srcfile/$mirrorsite -P /tmp/ 2>/dev/null >/dev/null
                tar xvfz /tmp/$srcfile -C /tmp >/dev/null
                rm -f /tmp/$srcfile >/dev/null
#                gprintf "MySQL $ver 소스가 /tmp/mysql-$ver 디렉토리에 설치되었습니다."; echo
#                gprintf ""
fi

# MySQL 컴파일 환경 설정
                gprintf "step3.MySQL 컴파일 환경을 준비하고 있습니다. 기다려 주세요.....!"; echo
cd /tmp/mysql-$ver
./configure --prefix=/usr/local/mysql --localstatedir=/usr/local/mysql/data --with-charset=utf8 --with-mysqld-user=mysql --sysconfdir=/etc --enable-thread-safe-client 2>/dev/null >/dev/null

if [ $? = "1" ]; then
    gprintf "컴파일 환경을 설정할 수 없습니다."; echo
    gprintf "이는 컴파일에 필요한 환경 조건이 구비되지 않았으므로"; echo
    gprintf "컴파일 환경 조건을 먼저 해결해 주시기 바랍니다."; echo
    gprintf " ./configure 명령을 실행하여 오류가 발생되는 부분을 처리해 주세요." ; echo
    exit 1
fi

# MySQL 컴파일
    gprintf "step4.MySQL를 컴파일하고 있습니다. 기다려 주세요.........!"; echo
make -j4 2>/dev/null >/dev/null
if [ $? = "1" ]; then
    gprintf "컴파일에서 오류가 발생되었습니다."; echo
    gprintf "make 명령으로 오류를 확인하시고 처리해 주세요."; echo
    exit 1
fi

# MySQL 컴파일 설치
    gprintf "step5.컴파일된 파일들을 설치하고 있습니다. 기다려 주세요.......!"; echo
make install 2>/dev/null >/dev/null

if [ $? = "1" ]; then
    gprintf "컴파일에서 오류가 발생되었습니다."; echo
    gprintf "make install 명령으로 오류를 확인하시고 처리해 주세요."; echo
    exit 1
fi

# MySQL 버전 기록
if [ ! -f /usr/local/mysql/.version ]; then
    echo $ver > /usr/local/mysql/.version
fi

# 컴파일후 MySQL 설정
## MySQL 계정 생성
    gprintf "step6.MySQL 설치 후 설정을 진행합니다."; echo

mysqldir=/usr/local/mysql
mysqluser=`grep mysql /etc/passwd | cut -d":" -f1 | awk '{print $1}'`

if [ mysql != "$mysqluser" ]; then
   adduser -d $mysqldir -s /sbin/nologin mysql
fi

## MySQL DB 경로 생성
if [ ! -d $mysqldir/data ]; then
    $mysqldir/bin/mysql_install_db
fi

## 소유권 조절
chown root.mysql -R $mysqldir/
chown mysql.mysql -R $mysqldir/data

## ld.so.conf
cat >> /etc/ld.so.conf <<EOT
/usr/local/mysql/lib
EOT

/sbin/ldconfig

## 설정 파일 복사
if [ ! -f /etc/my.cnf ]; then
   cp $mysqldir/share/mysql/my-large.cnf /etc/my.cnf
fi

## MySQL 데몬 띄우기

if [ -f $mysqldir/share/mysql/mysql.server ]; then
   cp $mysqldir/share/mysql/mysql.server /etc/init.d/mysqld
   chkconfig --level 3 mysqld on
   chkconfig --level 5 mysqld on
   /etc/init.d/mysqld restart
fi
gprintf "step7.MySQL-$ver이 /usr/local/mysql에 성공적으로 설치되었습니다."; echo
sleep 4

    clear
    gprintf "이제 마지막으로 PHP 소스를 설치합니다."; echo

#if [ -x $mysqldir/bin/mysqld_safe ]; then
#   $mysqldir/bin/mysqld_safe -user=mysql &
#fi

# 소스 삭제
rm -rf /tmp/mysql-$ver > /dev/null

## PHP Install
WEBREMOTEDIR=get
WEBHOST=kr2.php.net

gprintf "설치 가능한 PHP 소스 버전은 다음과 같습니다."; echo
gprintf "================"; echo
gprintf " PHP 소스 버전"; echo
gprintf "================"; echo
gprintf "     5.3.1      "; echo
gprintf "     5.2.13     "; echo
gprintf "================"; echo

echo -n "step1.설치하고자 하는 php 소스 버전을 선택해 주세요! "
read ver

if [ -z $ver ]; then
        echo -n "php 소스 버전을 입력하지 않으셨습니다. 다시 입력해 주세요! "
        read ver
        gprintf "php 소스 버전을 선택하지 않아 종료합니다." ; echo
        exit 1

        elif [ ! -z $ver ]; then
        gprintf "step2.PHP 소스를 /tmp/php-$ver 경로로 설치하고 있습니다."; echo
            srcfile=php-$ver.tar.bz2
            mirrorsite=from/this/mirror
            wget http://$WEBHOST/$WEBREMOTEDIR/$srcfile/$mirrorsite -P /tmp/ 2>/dev/null >/dev/null
            tar xvfj /tmp/$srcfile -C /tmp/ >/dev/null
            rm -f /tmp/$srcfile >/dev/null
#        gprintf "PHP $ver 소스가 /tmp/php-$ver 디렉토리에 설치되었습니다."; echo
fi

# 컴파일 환경 설정
        gprintf "step3.PHP 소스의 컴파일 환경을 설정하고 있습니다. 기다려 주세요........!"; echo
cd /tmp/php-$ver
./configure --with-apxs2=/usr/local/apache2/bin/apxs \
--with-mysql=/usr/local/mysql \
--with-config-file-path=/usr/local/apache2/conf \
--enable-safe-mode \
--enable-sockets \
--with-exec-dir=/usr/local/apache2/bin \
--with-gd=shared \
--with-openssl \
--enable-sysvsem \
--enable-sysvshm \
--enable-magic-quotes \
--enable-inline-optimization \
--enable-bcmath \
--enable-mbstring \
--enable-ftp \
--with-zlib \
--with-png-dir=/usr/lib \
--with-jpeg-dir=/usr/lib \
--with-libxml-dir=/usr/lib \
--with-gd \
--enable-gd-native-ttf \
--with-freetype-dir=/usr \
--with-gettext \
--enable-exif  2>/dev/null >/dev/null

# 컴파일
    gprintf "step4.PHP를 컴파일하고 있습니다. 기다려 주세요......"; echo
make -j4 2>/dev/null >/dev/null

# 컴파일 설치
    clear
    gprintf "step5.컴파일한 PHP 파일을 설치하고 있습니다."; echo
make install 2>/dev/null >/dev/null

gprintf "step6.PHP 설정을 하고 있습니다."; echo

# PHP 버전 기록
if [ ! -f /usr/local/lib/php/.version ]; then
    echo $ver > /usr/local/lib/php/.version
fi

# 설정 파일 복사
if [ ! -f /usr/local/apache2/conf/php.ini ]; then
   cp /tmp/php-$ver/php.ini-recommended /usr/local/apache2/conf/php.ini
fi
# 소스 삭제
rm -rf /tmp/php-$ver >/dev/null

# php addtype 설정
cat >> /usr/local/apache2/conf/httpd.conf <<EOT
AddType application/x-httpd-php .php
EOT
# phpinfo 
cat >> /usr/local/apache2/htdocs/test.php <<EOT
<?php phpinfo(); ?>
EOT
sleep 4

    clear
    gprintf "이제까지 모든 AMP 소스 설치가 성공적으로 완료되었습니다."; echo
    gprintf "수고하셨습니다....."; echo
    gprintf "##############################"; echo
    gprintf "리눅스를 더 가까이 "; echo
    gprintf "##############################"; echo
    gprintf "http://no1linux.org"; echo
    gprintf "##############################"; echo
